from jabberwock.ccm.abstracts import BaseXType


class XPhoneLine(BaseXType):
    pass


class XCallForwardAll(BaseXType):
    pass


class XCallForwardAlternateParty(BaseXType):
    pass


class XCallForwardBusy(BaseXType):
    pass


class XCallForwardBusyInt(BaseXType):
    pass


class XCallForwardNoAnswer(BaseXType):
    pass


class XCallForwardNoAnswerInt(BaseXType):
    pass


class XCallForwardNoCoverage(BaseXType):
    pass


class XCallForwardNoCoverageInt(BaseXType):
    pass


class XCallForwardNotRegistered(BaseXType):
    pass


class XCallForwardNotRegisteredInt(BaseXType):
    pass


class XCallForwardOnFailure(BaseXType):
    pass


class XLineAppearanceAssociationForPresence(BaseXType):
    pass


class XLineAssociation(BaseXType):
    pass


class XEnduserMember(BaseXType):
    pass


class XPhone(BaseXType):
    pass
